// XtraBtns.CC  -  Extra buttons
// By Peter Rorlick 
***********************************************************************
CLASS POPBUTTON(FormObj) OF FlashBUTTON(FormObj) Custom
   this.Text = "PopButton"
EndClass
******************************************************************
*
* FLASHBTN.CC
*
* Reverses the color of the pushbutton when the mouse
* cursor passes over it.
*
* Author: Gary White  garyw@qni.com
*
* Last modified: Oct 06, 1997
* Then by Peter Rorlick on March 2, 1998
*
* Usage: In the form designer, click File, Set up custom controls
* and add the FlashButton.CC.  Then just select the FlashButton
* from the Custom tab of your control palette and install it just
* like you would any other button.
*
* This custom class uses the pushbutton's OnOpen event to install
* the routines.  Do NOT overwrite the FlashButton's OnOpen event.
* The OnMouseMove event triggers the color change and is protected.
*
* Note that any controls created after the form's OnOpen will not
* have the routine installed to return the FlashButton to it's
* normal colors.  This should rarely be a problem unless the newly
* created control is touching the FlashButton, in which case the
* button will not return to normal until the mouse pointer moves
* off of the new control.  Unlikely, but it could happen.
*
* Thanks to Romain Strieff and Peter Rorlick for finding my errors
* and suggesting fixes and improvements.
*
* Enjoy!
* Gary
*
/*
Changes made by Peter Rorlick on March 2nd:

We now test whether a control is a Flashbutton or one of its
subclasses:
   if Type('oControl.HotColor')=='U'
This allows us to subclass it without it breaking.

I've modified the Do...Until loop so that it recurses through
the objects that belong to containers and notebooks.
This solves the small glitch that occurred when a
flashbutton resided on a container or notebook and there was
another control touching it: when the mouse moved from the
flashbutton to the other control, the button did not un-flash.

The March 2 version now sets a specific hot color (Black/Yellow) and
changes the borderstyle, instead of simply reversing the color.
I find that it looks much nicer. But if you have small buttons with
bitmaps, you might prefer to comment out the lines that change the
borderstyle (or set borderStyle to 4 rather than 8).

We now use an Init method instead of OnOpen, so that we don't have
to worry about programmers over-writing the OnOpen.  To take advantage of
the Init method, the form must be based on zForm or one of its subclasses.
If not based on zForm, it uses the OnOpen method.

The button also lights up now when the user tabs to it.
*/
*******

CLASS FLASHBUTTON( FormObj, Name ) OF PUSHBUTTON( FormObj, Name ) Custom
   protect OnOpen, OnMouseMove
   this.Height      = 1.3
   this.Width       = 12
   this.Text        = "FlashBtn"
   this.FontBold    = .f.
   this.OnMouseMove = class::FlashButton_OnMouseMove
   this.OnGotFocus  = class::FlashButton_OnMouseMove
   If Type('form.Init') # 'U'
     * If the form is subclassed from zForm, it will automatically
     * run any Init methods of the controls on the form, before the form
     * opens.  In this case, We use the Init method instead of the OnOpen,
     * to "free up" the button's OnOpen for other purposes that may be
     * desired by the programmer.
   Else
     * Form is not subclassed from zForm. In this case, remember to call
     * Flashbutton::Init() when you over-write the button's OnOpen.
     this.OnOpen = class::Init
   Endif
   this.parent.OnMouseMove = {;class::FlashButton_BlinkOff()}  // For Containers and Notebooks that contain flashbuttons.
   form.OnMouseMove = {;class::FlashButton_BlinkOff()}   // Needed when the only flashbuttons on the form are on Containers or Notebooks.


   Procedure FlashButton_OnMouseMove( flags, row, col )
      * Make the button stand out visually:
      if type("form.HotButton") # "O"
        this.ColorNormal = this.HotColor
        this.borderStyle = 8   //    4 is Single, 8 is modal
        form.HotButton = this
      elseif this.Name # form.HotButton.Name
        form.HotButton.BorderStyle = 0 // back to default
        form.HotButton.ColorNormal=form.HotButton.NormalColor
        this.borderStyle = 8
        this.ColorNormal = this.HotColor
        form.HotButton = this
      endif


   Procedure FlashButton_BlinkOff
      * Return the button's appearance to normal (un-flash it):
      if type("form.HotButton") == "O" and form.HotButton#this
         form.HotButton.ColorNormal=form.HotButton.NormalColor
         form.HotButton.BorderStyle = 0 // back to default
         form.HotButton=""
      endif

***
* The Init (OnOpen) routine installs the BlinkOff routine in all other
* controls on the form.  It also determines the NormalColor from
* the ColorNormal assigned by the programmer.
***

   Procedure Init  // was: FlashButton_OnOpen
      private oControl
      this.NormalColor=this.ColorNormal
      * this.HotColor=right(this.ColorNormal, len(this.ColorNormal) - ;
         rat("/", this.ColorNormal)) +;
         "/" + left(this.ColorNormal, at("/", this.ColorNormal) -1)
      this.HotColor = 'Black/Yellow'
      if type("form.FlashBtnInstalled") == "U"
         form.FlashBtnInstalled=.t.
         CLASS::Install_BlinkOff(Form)
      endif


   Proc Install_BlinkOff(InitialObject)
   private o
   o = InitialObject.First
   do
     if ("CONTAINER"$o.CLASSNAME) OR ("NOTEBOOK"$o.CLASSNAME)
       CLASS::Install_BlinkOff(o)   // recurse through the objects of the container or notebook
     EndIf
     if type( "o.onMouseMove" ) # "U"
       *if upper(o.ClassName) # "FLASHBUTTON"   // This line breaks when FlashButton is subclassed.
       if Type('o.HotColor')=='U'   // If this is NOT a flashbutton or one of its subclasses...
         if empty(o.OnMousemove)
           o.OnMouseMove = {;class::FlashButton_BlinkOff()}
         else
           o.sOnMouseMove = o.OnMouseMove
           o.FlashButton_Blinkoff = {;class::FlashButton_Blinkoff(); this.sOnMouseMove()}
           o.OnMouseMove = {;this.FlashButton_Blinkoff()}
         endif
       endif
     endif
     o = o.before
   until o.name == InitialObject.First.name
   return

ENDCLASS
***********************************************************************
CLASS OKPopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
     text = "O&K"
     SpeedTip = "Click this button to save changes and close the form"
     StatusText = "Save changes and close the form."
     UpBitmap = "RESOURCE #36"
     Height = 1.3
     Width = 12
   endwith
EndClass
***********************************************************************
CLASS CancelPopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
     text="&Cancel"
     SpeedTip="Click this button to discard changes and close the form"
     StatusText="Discard changes and close the form."
     UpBitmap="RESOURCE #28"
   endwith
EndClass
***********************************************************************
CLASS ClosePopBUTTON(FormObj) OF POPBUTTON(FormObj) Custom
   with (this)
     OnClick={;Form.Close()}
     text="&Close"
     SpeedTip="Click this button to close the form"
     StatusText="Close the form."
     UpBitmap="RESOURCE #1005"
   endwith
EndClass
***********************************************************************
// SButton is a small square button that has no text and does not retain focus.
CLASS SBUTTON(FormObj) OF PUSHBUTTON(FormObj) Custom
	With (this)
      height = 1.25
      width = 3.55
      metric = 0
      tabStop = false
      group = true
      value = false
      text = " "
      SpeedBar = true
   EndWith
EndClass

